/*
 * app.c
 *
 *  Created on: Aug 12, 2020
 *      Author: kaige
 */

#include <stdio.h>
#include "ucos_ii.h"
#include "ucos_bsp.h"
#include "xipipsu.h"
#include "xscugic.h"

#include <ucos_int.h>

#include "lwip/tcpip.h"
#include "lwip/dhcp.h"

#include "lwip/init.h"
#include "lwip/inet.h"
#include "lwip/opt.h"
#include "lwip/ip_addr.h"
#include "lwipopts.h"
#include "xil_mmu.h"

#include "lwip/sockets.h"

#include "ILM/if/ilm_core.h"

#include "xil_cache.h"

#define TICKS_PER_MACRO_PERIOD 	100	//10Hz PWM, 20ms

extern const ILM_LET_TABLE_T CO_LET_TABLE[];
extern const size_t CO_LET_TABLE_SIZE;

#include "imw_data_writer.h"
#include "imw_data_reader.h"

#include "imw_example_configs.h"



static CPU_STK imwLetTask3Stk[IMW_LET_TASK_3_STACK_SIZE];

XIpiPsu ILM_IPIInstance;
extern XIpiPsu_Config XIpiPsu_ConfigTable[XPAR_XIPIPSU_NUM_INSTANCES];


IMW_ERR_t dataReader3Function(void* p_arg);

IMW_OBJECT_t imwObject1 = {
		.objectId=IMW_OBJECT_1_ID,
		.size=IMW_OBJECT_1_SIZE,
		.historyCount=2,
		.baseAddr=(void*)IMW_OBJECT_1_BASE
};

IMW_OBJECT_t imwObject2 = {
		.objectId=IMW_OBJECT_2_ID,
		.size=IMW_OBJECT_2_SIZE,
		.historyCount=2,
		.baseAddr=(void*)IMW_OBJECT_2_BASE
};

IMW_DATA_READER_INTERNAL_t imwDataReader3Internal;

IMW_DATA_READER_CONFIG_t imwDataReader3Config = {
		.taskStack = (void*)imwLetTask3Stk,
		.taskPrio = IMW_LET_TASK_3_PRIO,
		.taskStackSize = IMW_LET_TASK_3_STACK_SIZE,
		.taskId =IMW_LET_TASK_3_ID,
		.function = dataReader3Function,
		.internal = &imwDataReader3Internal,
};

const uint8_t dataReader3Mapping[] = {1,1,0,0};

IMW_ERR_t dataReader3Function(void* p_arg){
	IMW_DATA_READER_INTERNAL_t *internal;
	IMW_DATA_READER_CONFIG_t *config = (IMW_DATA_READER_CONFIG_t*)p_arg;

	IMW_ARG_NULL_CHECK(config);
	IMW_ARG_NULL_CHECK(config->internal);

	internal = config->internal;

	void *historyBufferEntry1 = imwObject1.baseAddr + ((dataReader3Mapping[internal->currentSeqNr % (sizeof(dataReader3Mapping)/sizeof(uint8_t))]) * imwObject1.size);

	void *historyBufferEntry2 = imwObject2.baseAddr + ((internal->currentSeqNr % imwObject2.historyCount) * imwObject2.size);

	memcpy(historyBufferEntry2,historyBufferEntry1,imwObject2.size);

	return IMW_ERR_NONE;
}

void ILM_ISR(void*arg){
	u32 IpiSrcMask; /**< Holds the IPI status register value */
	IpiSrcMask = XIpiPsu_GetInterruptStatus(&ILM_IPIInstance);
	ILM_Handler(0);
	XIpiPsu_ClearInterruptStatus(&ILM_IPIInstance,IpiSrcMask);
}

extern void ILM_OS_RegisterActSem(uint8_t TaskId, OS_EVENT* actSem);

#define NR_WAYS   	4
#define NR_SETS  	128
#define LINE_SIZE 	(64/4)

uint32_t cachableObject[32/4*1024];
void accessCache(){
	int set = 0;
	int way = 0;
	for(set = 0; set < NR_SETS; set++){
		for(way = 0; way < NR_WAYS; way++){
			Xil_DCacheFlushLine(&cachableObject[set * way * LINE_SIZE]);
		}
	}
}

#define __burn_wcet(x,y)                \
    ({ CPU_INT32U __mul = (x);                        \
       CPU_INT32U __scale = (y);                      \
       asm volatile(	"		mov x0,#0		\n\t" \
						"		mul x1,%0,%1	\n\t" \
    					"1: 	add x0,x0,#1	\n\t" \
    					"		cmp x1,x0		\n\t" \
    					" 		bne 1b 			\n\t" \
    		   	   : : "r" (__mul) , "r" (__scale) : ); })

extern void traceHook(uint32_t hookN, uint32_t hookP1, uint32_t hookP2);

static OS_STK dummyLoadTaskStack[1024];
static void dummyLoadTask(void *p_arg) {
	uint8_t err;
	OS_EVENT *sem;

	/* Initialize IPI IRQ for activation */
	sem = OSSemCreate(0);
	ILM_OS_RegisterActSem(9, sem);

	while(1){
		OSSemPend(sem,0,&err);
		uint32_t wcet = 300 + rand() % (400+1 - 300);
		traceHook(0xABCDE, 1 , wcet);
		__burn_wcet(wcet,600);
		traceHook(0xABCDE, 2 , wcet);
		accessCache();
		traceHook(0xABCDE, 3 , wcet);
	}
}

extern void tcpTokenRingInit(void);

void startAPP(void){
	uint8_t err = 0;
	Xil_SetTlbAttributes(IMW_OBJECT_1_BASE, NORM_NONCACHE);

	tcpTokenRingInit();

	ILM_Init((ILM_LET_TABLE_T *) CO_LET_TABLE, CO_LET_TABLE_SIZE,TICKS_PER_MACRO_PERIOD);

	err = OSTaskCreateExt( dummyLoadTask,
				(void*)0,
				&dummyLoadTaskStack[1024-1],
				1,
				1,
				dummyLoadTaskStack,
				1024,
				 DEF_NULL,
				(OS_TASK_OPT_STK_CLR | OS_TASK_OPT_STK_CHK));

	OSTaskNameSet(1,(INT8U *)"dummyLoadTask",NULL);

	XIpiPsu_Config *_XIpiPsu_ConfigTable = &XIpiPsu_ConfigTable[XPAR_PSU_IPI_7_DEVICE_ID];
	XIpiPsu_CfgInitialize(&ILM_IPIInstance, _XIpiPsu_ConfigTable, _XIpiPsu_ConfigTable->BaseAddress);
	XIpiPsu_InterruptEnable(&ILM_IPIInstance, XIPIPSU_ALL_MASK);
	XIpiPsu_ClearInterruptStatus(&ILM_IPIInstance, XIPIPSU_ALL_MASK);
	UCOS_IntVectSet(_XIpiPsu_ConfigTable->IntId, 0, 0, ILM_ISR, DEF_NULL);
	UCOS_IntSrcEn(_XIpiPsu_ConfigTable->IntId);

	imw_dataReader_create(&imwDataReader3Config);

	return;
}
