/*
*********************************************************************************************************
*                                            EXAMPLE CODE
*
*               This file is provided as an example on how to use Micrium products.
*
*               Please feel free to use any application code labeled as 'EXAMPLE CODE' in
*               your application products.  Example code may be used as is, in whole or in
*               part, or may be used as a reference only. This file can be modified as
*               required to meet the end-product requirements.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*
*               You can find our product's user manual, API reference, release notes and
*               more information at https://doc.micrium.com.
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                             uC/OS-II
*                                         Application Hooks
*
* Filename      : app_hooks.c
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include  <ucos_ii.h>
#include "imw_types.h"
#include "ILM/if/ilm_hooks.h"
#include "ptpd.h"
#include "imw_example_configs.h"
#include "imw_header.h"
#include "ida-lwip/ida_lwip_filter.h"
#include "netif/xemacps_ieee1588.h"

/*
*********************************************************************************************************
*                                       EXTERN  GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                           LOCAL CONSTANTS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/



/*
**********************************************************************************************************
**********************************************************************************************************
**                                        GLOBAL FUNCTIONS
**********************************************************************************************************
**********************************************************************************************************
*/

volatile __attribute__ ((noinline)) void stopTest(){
	asm volatile("nop");
}

static volatile int running __attribute__ ((unused)) = 0;

volatile int counter = 0;
static struct ptptime_t timestamp;

#define STM_TRACE_BASE	0xF8000000
#define STM_TRACE(id, value)	({ \
	asm volatile("": : :"memory"); \
	*(volatile uint32_t*)(STM_TRACE_BASE + (((uint16_t)id) * 0x100)) = (uint32_t)(value); \
	asm volatile("": : :"memory"); \
})
#define __TRACE_2X16_TO_32(upper, lower) (uint32_t)((((uint32_t)upper) << 16) | (((uint32_t)lower) & 0xFFFF))
#define STM_TRACE_2x16(id, upper, lower)	STM_TRACE(id, __TRACE_2X16_TO_32(upper, lower)) 

void intr_save_regs_hook(u32 RegISR, u32 RegQ1ISR, u32 RegNWCTR) {
//	traceHook(0xC, 0, 0);
//	traceHookRaw(RegISR);
//	traceHookRaw(RegQ1ISR);
//	traceHookRaw(RegNWCTR);
//	traceHook(0xC, 0xFFF, 0xFFFF);
	return;
}

void imw_writer_start_hook(IMW_WRITER_CONFIG_t *p_arg) {
	STM_TRACE_2x16(0x2, p_arg->object->objectId, p_arg->internal->currentObjectSeqNr);
	return;
}

void imw_writer_end_hook(IMW_WRITER_CONFIG_t *p_arg) {
	STM_TRACE_2x16(0x3, p_arg->object->objectId, p_arg->internal->currentObjectSeqNr);
	counter++;
#if BOARD == 1
#ifdef IN_CHUNKS
#ifdef TEST2
	/* 50s * 6 Chunks/10ms + 2 Chunks/10ms */
	if(counter == 40000){
		stopTest();
	}
#else
	/* 50s * 6 Chunks/10ms */
	if(counter == 30000){
		stopTest();
	}
#endif
#else
	/* 50s * 1 Chunk/10ms */
	if(counter == 5000){
		stopTest();
	}
#endif
#else
	/* 50s * 1 Chunk/5ms */
	if(counter == 10000){
		stopTest();
	}
#endif
	return;
}

uint32_t writerSeqNr = 0;

void ILM_HOOK_TACT(ILM_LET_TABLE_T *pEntry){
	if(!running)
		running = 1;
#if BOARD==1
	if(pEntry->TaskId == IMW_WRITER_TASK_1_ID){
#else
	if(pEntry->TaskId == IMW_WRITER_TASK_2_ID){
#endif
		STM_TRACE_2x16(0x4, pEntry->TaskId, writerSeqNr);
		writerSeqNr++;
	}
	return;
}

void ILM_HOOK_TINCR(ILM_LET_TABLE_T *pEntry){
	STM_TRACE(0x5, *((uint16*)(pEntry->Payload.pSchedEvent))-1);
}

void imw_reader_sumbessageData_start_hook(IMW_SUBMESSAGE_DATA_HEADER_t *header) {
	if(header->objectId != IMW_OBJECT_3_ID)
		STM_TRACE_2x16(0x6, header->objectSequenceNr, header->fragmentId);
	return;
}

void imw_reader_sumbessageData_end_hook(IMW_SUBMESSAGE_DATA_HEADER_t *header) {
	if(header->objectId != IMW_OBJECT_3_ID)
		STM_TRACE_2x16(0x7, header->objectSequenceNr, header->fragmentId);
	return;
}

void imw_reader_bitmapCheckFailed_hook(IMW_READER_CONFIG_t *p_arg) {
	STM_TRACE(0x8, 0);
	return;
}

void emacps_send_handler_hook(u32 x, u32 y) {
//	traceHook(0xF, y, x);
	return;
}

void emac_reset_hook(void) {
//	traceHook(0xF, 0xFFFF, 0xFFF);
	return;
}

void emacps_send_hook(u32 x, u32 y) {
	STM_TRACE_2x16(0xD,x,y);
	return;
}

void emacps_tx_hook(struct pbuf* p) {
	struct eth_hdr *ethhdr;
	struct ip_hdr *iphdr;
	IMW_SUBMESSAGE_DATA_HEADER_t *imw_data_header;

	if(p->tot_len == 1086){
		ethhdr = (struct eth_hdr *)p->payload;
		if(ethhdr->type == PP_HTONS(ETHTYPE_VLAN)){
			if(p->len < p->tot_len && p->next != (struct pbuf*)0){
				imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->next->payload + sizeof(IMW_HEADER_t));
				STM_TRACE_2x16(0x9,imw_data_header->objectSequenceNr, imw_data_header->fragmentId); 
			} else {
				iphdr = (struct ip_hdr*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR);
				uint32_t len = lwip_ntohs(IPH_LEN(iphdr));
				uint32_t iphdr_hlen = IPH_HL_BYTES(iphdr);
				if(len == 1068){
					imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR + iphdr_hlen + UDP_HLEN + sizeof(IMW_HEADER_t));
					STM_TRACE_2x16(0x9,imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
				}
			}
		}
	}
	return;
}

void emacps_tx_complete_hook(struct pbuf* p) {
	struct eth_hdr *ethhdr;
	struct ip_hdr *iphdr;
	IMW_SUBMESSAGE_DATA_HEADER_t *imw_data_header;
	if(p->tot_len == 1086){
		ethhdr = (struct eth_hdr *)p->payload;
		if(ethhdr->type == PP_HTONS(ETHTYPE_VLAN)){
			if(p->len < p->tot_len && p->next != (struct pbuf*)0){
				imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->next->payload + sizeof(IMW_HEADER_t));
				STM_TRACE_2x16(0xA, imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
			} else {
				iphdr = (struct ip_hdr*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR);
				uint32_t len = lwip_ntohs(IPH_LEN(iphdr));
				uint32_t iphdr_hlen = IPH_HL_BYTES(iphdr);
				if(len == 1068){
					imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR + iphdr_hlen + UDP_HLEN + sizeof(IMW_HEADER_t));
					STM_TRACE_2x16(0xA, imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
				}
			}
		}
	}
	return;
}

void emacps_rx_hook(struct pbuf* p) {
	struct eth_hdr *ethhdr;
	struct ip_hdr *iphdr;
	IMW_SUBMESSAGE_DATA_HEADER_t *imw_data_header;
	if(p->tot_len == 1086){
		ethhdr = (struct eth_hdr *)p->payload;
		if(ethhdr->type == PP_HTONS(ETHTYPE_VLAN)){
			iphdr = (struct ip_hdr*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR);
			uint32_t len = lwip_ntohs(IPH_LEN(iphdr));
			uint32_t iphdr_hlen = IPH_HL_BYTES(iphdr);
			if(len == 1068){
				imw_data_header = (IMW_SUBMESSAGE_DATA_HEADER_t*)(p->payload + SIZEOF_ETH_HDR + SIZEOF_VLAN_HDR + iphdr_hlen + UDP_HLEN + sizeof(IMW_HEADER_t));
				STM_TRACE_2x16(0xB, imw_data_header->objectSequenceNr, imw_data_header->fragmentId);
			}
		}
	}
	return;
}

void tcpReadHook(int tcpIteration, size_t bytesLeft){
	//traceHook(0x0000000C, (uint32_t)tcpIteration, (uint32_t)bytesLeft);
}

void tcpWriteHook(int tcpIteration, size_t bytesLeft){
	//traceHook(0x0000000D, (uint32_t)tcpIteration, (uint32_t)bytesLeft);
}

void imw_writer_send_fragment_hook(uint16_t seqNr, int fragment) {
	//traceHook(0x0000000E, seqNr, fragment);
	return;
}

void errorHook(uint32_t err1, uint32_t err2){
	STM_TRACE_2x16(0xF, err1, err2);
}

void ETH_PTP_SET_INCR_HOOK(uint32_t ns, uint32_t sub_ns){
	//traceHook(0xE, ns, sub_ns);
}

/*
*********************************************************************************************************
*********************************************************************************************************
**                                         uC/OS-II APP HOOKS
*********************************************************************************************************
*********************************************************************************************************
*/

#if (OS_APP_HOOKS_EN > 0)

/*
*********************************************************************************************************
*                                      TASK CREATION HOOK (APPLICATION)
*
* Description : This function is called when a task is created.
*
* Argument(s) : ptcb   is a pointer to the task control block of the task being created.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*********************************************************************************************************
*/

void  App_TaskCreateHook (OS_TCB *ptcb)
{
#if (APP_CFG_PROBE_OS_PLUGIN_EN == DEF_ENABLED) && (OS_PROBE_HOOKS_EN > 0)
    OSProbe_TaskCreateHook(ptcb);
#endif
}

/*
*********************************************************************************************************
*                                    TASK DELETION HOOK (APPLICATION)
*
* Description : This function is called when a task is deleted.
*
* Argument(s) : ptcb   is a pointer to the task control block of the task being deleted.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*********************************************************************************************************
*/

void  App_TaskDelHook (OS_TCB *ptcb)
{
    (void)ptcb;
}

/*
*********************************************************************************************************
*                                      IDLE TASK HOOK (APPLICATION)
*
* Description : This function is called by OSTaskIdleHook(), which is called by the idle task.  This hook
*               has been added to allow you to do such things as STOP the CPU to conserve power.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts are enabled during this call.
*********************************************************************************************************
*/

#if OS_VERSION >= 251
void  App_TaskIdleHook (void)
{
	CPU_SR cpu_sr;
	OS_ENTER_CRITICAL();
	if(running)
		STM_TRACE(0x0, 0);
#ifdef WFI_IDLE
    __asm volatile( "dsb" );
    __asm volatile( "wfi" );
    __asm volatile( "isb" );
#endif
	if(running)
		STM_TRACE(0x1, 0);
    OS_EXIT_CRITICAL();
}
#endif

/*
*********************************************************************************************************
*                                        STATISTIC TASK HOOK (APPLICATION)
*
* Description : This function is called by OSTaskStatHook(), which is called every second by uC/OS-II's
*               statistics task.  This allows your application to add functionality to the statistics task.
*
* Argument(s) : none.
*********************************************************************************************************
*/

void  App_TaskStatHook (void)
{
}

/*
*********************************************************************************************************
*                                            TASK RETURN HOOK (APPLICATION)
*
* Description: This function is called if a task accidentally returns.  In other words, a task should
*              either be an infinite loop or delete itself when done.
*
* Arguments  : ptcb      is a pointer to the task control block of the task that is returning.
*
* Note(s)    : none
*********************************************************************************************************
*/


#if OS_VERSION >= 289
void  App_TaskReturnHook (OS_TCB  *ptcb)
{
    (void)ptcb;
}
#endif

/*
*********************************************************************************************************
*                                        TASK SWITCH HOOK (APPLICATION)
*
* Description : This function is called when a task switch is performed.  This allows you to perform other
*               operations during a context switch.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*
*               (2) It is assumed that the global pointer 'OSTCBHighRdy' points to the TCB of the task that
*                   will be 'switched in' (i.e. the highest priority task) and, 'OSTCBCur' points to the
*                  task being switched out (i.e. the preempted task).
*********************************************************************************************************
*/

#if OS_TASK_SW_HOOK_EN > 0
void  App_TaskSwHook (void)
{
#if (APP_CFG_PROBE_OS_PLUGIN_EN > 0) && (OS_PROBE_HOOKS_EN > 0)
    OSProbe_TaskSwHook();
#endif
    STM_TRACE_2x16(0xE, OSTCBCur->OSTCBPrio, OSTCBHighRdy->OSTCBPrio);
}
#endif

/*
*********************************************************************************************************
*                                     OS_TCBInit() HOOK (APPLICATION)
*
* Description : This function is called by OSTCBInitHook(), which is called by OS_TCBInit() after setting
*               up most of the TCB.
*
* Argument(s) : ptcb    is a pointer to the TCB of the task being created.
*
* Note(s)     : (1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/

#if OS_VERSION >= 204
void  App_TCBInitHook (OS_TCB *ptcb)
{
    (void)ptcb;
}
#endif

/*
*********************************************************************************************************
*                                        TICK HOOK (APPLICATION)
*
* Description : This function is called every tick.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/

#if OS_TIME_TICK_HOOK_EN > 0
void  App_TimeTickHook (void)
{
#if (APP_CFG_PROBE_OS_PLUGIN_EN == DEF_ENABLED) && (OS_PROBE_HOOKS_EN > 0)
    OSProbe_TickHook();
#endif
	STM_TRACE(0x10, 0);
}
#endif
#endif
