/*
 * app.c
 *
 *  Created on: Aug 12, 2020
 *      Author: kaige
 */

#include <stdio.h>
#include "cpu_core.h"
#include "ucos_ii.h"
#include "ucos_bsp.h"
#include <ucos_int.h>
#include "xil_types.h"
#include "xreg_cortexr5.h"
#include "xil_mpu.h"
#include "string.h"

#define MEMCPY_SIZE		(64*1024)

#define DRAM_MEM_BASE	0x70000000
#define DRAM_MEM_SIZE	0x10000000

#define SRAM_MEM_SIZE	(MEMCPY_SIZE)

#define OCM_MEM_BASE	0xFFFC0000
#define OCM_MEM_SIZE	0x00040000

volatile uint8_t tcm_test_mem[SRAM_MEM_SIZE] __attribute__ ((aligned (4))) __attribute__ ((section (".psu_lwip_tcm")));


#define STM_TRACE_BASE	0xF8000000
#define STM_TRACE(id, value)	*(volatile uint32_t*)(STM_TRACE_BASE + ((uint16_t)id * 0x100)) = (uint32_t)(value);

#pragma GCC push_options
#pragma GCC optimize ("O0")

void doMemcpy(void* targetMem, void* sourceMem, int iterations, uint32_t traceID){
	for(int i = 0; i < iterations; i++){
		STM_TRACE(0x2000, 0);
		memcpy(targetMem, sourceMem, MEMCPY_SIZE);
		dsb();
		STM_TRACE(traceID, 0);
	}
}

void startAPP(void){
	void* dramMemBase = (void*)(DRAM_MEM_BASE);
	void* dramMemBase2 = (void*)(DRAM_MEM_BASE + MEMCPY_SIZE);
	void* ocmMemBase = (void*)(OCM_MEM_BASE);
	void* ocmMemBase2 = (void*)(OCM_MEM_BASE + MEMCPY_SIZE);
	uint32_t i = 0, j = 0;
	CPU_SR cpu_sr;
	
	Xil_SetMPURegion((uint32_t)dramMemBase, DRAM_MEM_SIZE, DEVICE_NONSHARED | PRIV_RW_USER_RW);
	Xil_SetMPURegion((uint32_t)tcm_test_mem, SRAM_MEM_SIZE, DEVICE_NONSHARED | PRIV_RW_USER_RW);
	Xil_SetMPURegion((uint32_t)ocmMemBase, OCM_MEM_SIZE, DEVICE_NONSHARED | PRIV_RW_USER_RW);
	
	OS_ENTER_CRITICAL();
	
	while(1){
		/* Initialize memory */
		for(i = 0; i < SRAM_MEM_SIZE; i++){
			tcm_test_mem[i] = (uint8_t)(i + j);
		}
		dsb();
		
		/* First copy SRAM to DRAM */
		doMemcpy(dramMemBase,(void*)tcm_test_mem, j, 0x2001);
		
		/* Then copy DRAM to DRAM */
		doMemcpy(dramMemBase2, dramMemBase, j, 0x2002);
		
		/* Then copy SRAM to OCM */
		doMemcpy(ocmMemBase,(void*)tcm_test_mem, j, 0x2003);
		
		/* Then copy OCM to OCM */
		doMemcpy(ocmMemBase2, ocmMemBase, j, 0x2004);
		
		/* Then copy OCM to DRAM */
		doMemcpy(dramMemBase, ocmMemBase, j, 0x2005);
		
		/* Next run */
		j = (j + 1) % 10;
	}
	
	OS_EXIT_CRITICAL();
	
	return;
}

#pragma GCC pop_options
